import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="kfwx",
    version="0.0.8",
    author="Tiffany Nguyen",
    description="A data analysis toolkit for the KFW lab (MacOS X)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/tiffkwin/KFW",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: MacOS :: MacOS X",
    ],
)
